from django import template
from django.urls import NoReverseMatch, reverse

from misago.conf import settings

register = template.Library()


@register.simple_tag
def absoluteurl(url_or_name, *args, **kwargs):
    if not settings.MISAGO_ADDRESS:
        return None

    absolute_url_prefix = settings.MISAGO_ADDRESS.rstrip('/')

    try:
        url_or_name = reverse(url_or_name, args=args, kwargs=kwargs)
    except NoReverseMatch:
        # don't use URLValidator because its too explicit
        if not url_or_name.startswith('/'):
            return url_or_name
    
    return u'{}{}'.format(absolute_url_prefix, url_or_name)


from __future__ import unicode_literals

from datetime import timedelta

from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand
from django.utils import timezone

from misago.conf import settings
from misago.core.pgutils import chunk_queryset


UserModel = get_user_model()


class Command(BaseCommand):
    help = (
        "Deletes inactive user accounts older than set time."
    )

    def handle(self, *args, **options):
        if not settings.MISAGO_DELETE_NEW_INACTIVE_USERS_OLDER_THAN_DAYS:
            self.stdout.write("Automatic deletion of inactive users is currently disabled.")
            return


        users_deleted = 0
        
        joined_on_cutoff = timezone.now() - timedelta(
            days=settings.MISAGO_DELETE_NEW_INACTIVE_USERS_OLDER_THAN_DAYS)

        queryset = UserModel.objects.filter(
            requires_activation__gt=UserModel.ACTIVATION_NONE,
            joined_on__lt=joined_on_cutoff,
        )

        for user in chunk_queryset(queryset):
            user.delete()
            users_deleted += 1

        self.stdout.write("Deleted users: {}".format(users_deleted))



from django.contrib import messages
from django.utils.translation import ugettext_lazy as _

from misago.admin.views import generic
from misago.users.datadownloads import (
    expire_user_data_download, request_user_data_download, user_has_data_download_request)
from misago.users.forms.admin import RequestDataDownloadsForm, SearchDataDownloadsForm
from misago.users.models import DataDownload


class DataDownloadAdmin(generic.AdminBaseMixin):
    root_link = 'misago:admin:users:data-downloads:index'
    templates_dir = 'misago/admin/datadownloads'
    model = DataDownload


class DataDownloadsList(DataDownloadAdmin, generic.ListView):
    items_per_page = 30
    ordering = [
        ('-id', _("From newest")),
        ('id', _("From oldest")),
    ]
    selection_label = _('With data downloads: 0')
    empty_selection_label = _('Select data downloads')
    mass_actions = [
        {
            'action': 'expire',
            'name': _("Expire downloads"),
            'icon': 'fa fa-ban',
            'confirmation': _("Are you sure you want to set selected data downloads as expired?"),
        },
        {
            'action': 'delete',
            'name': _("Delete downloads"),
            'icon': 'fa fa-times-circle',
            'confirmation': _("Are you sure you want to delete selected data downloads?"),
        },
    ]

    def get_queryset(self):
        qs = super(DataDownloadsList, self).get_queryset()
        return qs.select_related('user', 'requester')
        
    def get_search_form(self, request):
        return SearchDataDownloadsForm

    def action_expire(self, request, data_downloads):
        for data_download in data_downloads:
            expire_user_data_download(data_download)

        messages.success(request, _("Selected data downloads have been set as expired."))

    def action_delete(self, request, data_downloads):
        for data_download in data_downloads:
            data_download.delete()

        messages.success(request, _("Selected data downloads have been deleted."))


class RequestDataDownloads(DataDownloadAdmin, generic.FormView):
    form = RequestDataDownloadsForm

    def handle_form(self, form, request):
        for user in form.cleaned_data['users']:
            if not user_has_data_download_request(user):
                request_user_data_download(user, requester=request.user)

        messages.success(request, _("Data downloads have been requested for specified users."))



from django.test import TestCase
from django.urls import reverse

from misago.legal.models import Agreement


class PrivacyPolicyTests(TestCase):
    def setUp(self):
        Agreement.objects.invalidate_cache()

    def tearDown(self):
        Agreement.objects.invalidate_cache()

    def test_404_on_no_policy(self):
        """policy view returns 404 when no policy is set"""
        response = self.client.get(reverse('misago:privacy-policy'))
        self.assertEqual(response.status_code, 404)

    def test_301_on_link_policy(self):
        """policy view returns 302 redirect when link is set"""
        Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='http://test.com',
            text='Lorem ipsum',
            is_active=True,
        )

        response = self.client.get(reverse('misago:privacy-policy'))
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['location'], 'http://test.com')

    def test_200_on_link_policy(self):
        """policy view returns 200 when custom tos content is set"""
        Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            title='Test Policy',
            text='Lorem ipsum dolor',
            is_active=True,
        )

        response = self.client.get(reverse('misago:privacy-policy'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Test Policy')
        self.assertContains(response, 'Lorem ipsum dolor')


class TermsOfServiceTests(TestCase):
    def setUp(self):
        Agreement.objects.invalidate_cache()

    def tearDown(self):
        Agreement.objects.invalidate_cache()

    def test_404_on_no_tos(self):
        """TOS view returns 404 when no TOS is set"""
        response = self.client.get(reverse('misago:terms-of-service'))
        self.assertEqual(response.status_code, 404)

    def test_301_on_link_tos(self):
        """TOS view returns 302 redirect when link is set"""
        Agreement.objects.create(
            type=Agreement.TYPE_TOS,
            link='http://test.com',
            text='Lorem ipsum',
            is_active=True,
        )

        response = self.client.get(reverse('misago:terms-of-service'))
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['location'], 'http://test.com')

    def test_200_on_link_tos(self):
        """TOS view returns 200 when custom tos content is set"""
        Agreement.objects.create(
            type=Agreement.TYPE_TOS,
            title='Test ToS',
            text='Lorem ipsum dolor',
            is_active=True,
        )

        response = self.client.get(reverse('misago:terms-of-service'))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Test ToS')
        self.assertContains(response, 'Lorem ipsum dolor')



from django.urls import reverse

from misago.admin.testutils import AdminTestCase
from misago.legal.models import Agreement


class AgreementAdminViewsTests(AdminTestCase):
    def test_link_registered(self):
        """admin nav contains agreements link"""
        response = self.client.get(reverse('misago:admin:users:accounts:index'))

        response = self.client.get(response['location'])
        self.assertContains(response, reverse('misago:admin:users:agreements:index'))

    def test_list_view(self):
        """agreements list view returns 200"""
        response = self.client.get(reverse('misago:admin:users:agreements:index'))
        self.assertEqual(response.status_code, 302)

        response = self.client.get(response['location'])
        self.assertEqual(response.status_code, 200)

    def test_mass_delete(self):
        """adminview deletes multiple agreements"""
        for i in range(10):
            response = self.client.post(
                reverse('misago:admin:users:agreements:new'),
                data={
                    'type': Agreement.TYPE_TOS,
                    'text': 'test agreement!',
                },
            )
            self.assertEqual(response.status_code, 302)

        self.assertEqual(Agreement.objects.count(), 10)

        agreements_pks = []
        for agreement in Agreement.objects.iterator():
            agreements_pks.append(agreement.pk)

        response = self.client.post(
            reverse('misago:admin:users:agreements:index'),
            data={
                'action': 'delete',
                'selected_items': agreements_pks,
            },
        )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Agreement.objects.count(), 0)

    def test_new_view(self):
        """new agreement view has no showstoppers"""
        response = self.client.get(reverse('misago:admin:users:agreements:new'))
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'Test Rules',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
            },
        )
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:users:agreements:index'))
        response = self.client.get(response['location'])
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Test Rules')

        test_agreement = Agreement.objects.get(type=Agreement.TYPE_TOS)
        self.assertIsNone(test_agreement.last_modified_on)
        self.assertIsNone(test_agreement.last_modified_by)
        self.assertIsNone(test_agreement.last_modified_by_name)

    def test_new_view_change_active(self):
        """new agreement view creates new active agreement"""
        response = self.client.get(reverse('misago:admin:users:agreements:new'))
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'Old Active',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
                'is_active': True,
            },
        )
        self.assertEqual(response.status_code, 302)

        response = self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'New Active',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
                'is_active': True,
            },
        )
        self.assertEqual(response.status_code, 302)

        test_agreement = Agreement.objects.get(is_active=True)
        self.assertEqual(test_agreement.title, 'New Active')

    def test_edit_view(self):
        """edit agreement view has no showstoppers"""
        self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'Test Rules',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
            },
        )

        test_agreement = Agreement.objects.get(type=Agreement.TYPE_TOS)
        form_link = reverse(
            'misago:admin:users:agreements:edit',
            kwargs={
                'pk': test_agreement.pk,
            },
        )

        response = self.client.post(
            form_link,
            data={
                'type': Agreement.TYPE_PRIVACY,
                'title': 'Test Privacy',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
            },
        )
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:users:agreements:index'))
        response = self.client.get(response['location'])
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Test Privacy')

        updated_agreement = Agreement.objects.get(type=Agreement.TYPE_PRIVACY)
        self.assertTrue(updated_agreement.last_modified_on)
        self.assertEqual(updated_agreement.last_modified_by, self.user)
        self.assertEqual(updated_agreement.last_modified_by_name, self.user.username)

    def test_edit_view_change_active(self):
        """edit agreement view sets new active"""
        self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'Old Active',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
                'is_active': True
            },
        )

        self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'New Active',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
            },
        )

        test_agreement = Agreement.objects.get(title='New Active')
        form_link = reverse(
            'misago:admin:users:agreements:edit',
            kwargs={
                'pk': test_agreement.pk,
            },
        )

        response = self.client.post(
            form_link,
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'Updated Active',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
                'is_active': True
            },
        )
        self.assertEqual(response.status_code, 302)

        updated_agreement = Agreement.objects.get(is_active=True)
        self.assertEqual(updated_agreement.title, 'Updated Active')

    def test_delete_view(self):
        """delete agreement view has no showstoppers"""
        self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'Test Rules',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
            },
        )

        test_agreement = Agreement.objects.get(type=Agreement.TYPE_TOS)

        response = self.client.post(
            reverse(
                'misago:admin:users:agreements:delete',
                kwargs={
                    'pk': test_agreement.pk,
                },
            )
        )
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:users:agreements:index'))
        self.client.get(response['location'])
        response = self.client.get(response['location'])

        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, test_agreement.title)

    def test_set_as_active_view(self):
        """set agreement as active view has no showstoppers"""
        self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'Test Rules',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
            },
        )

        test_agreement = Agreement.objects.get(type=Agreement.TYPE_TOS)

        response = self.client.post(
            reverse(
                'misago:admin:users:agreements:set-as-active',
                kwargs={
                    'pk': test_agreement.pk,
                },
            )
        )
        self.assertEqual(response.status_code, 302)

        updated_agreement = Agreement.objects.get(is_active=True)
        self.assertEqual(updated_agreement, test_agreement)

    def test_set_as_active_view_change_active(self):
        """set agreement as active view changes current active"""
        self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'Old Active',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
                'is_active': True,
            },
        )

        self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'title': 'New Active',
                'text': 'Lorem ipsum dolor met sit amet elit',
                'link': 'https://example.com/rules/',
            },
        )

        test_agreement = Agreement.objects.get(title='New Active')

        response = self.client.post(
            reverse(
                'misago:admin:users:agreements:set-as-active',
                kwargs={
                    'pk': test_agreement.pk,
                },
            )
        )
        self.assertEqual(response.status_code, 302)

        updated_agreement = Agreement.objects.get(is_active=True)
        self.assertEqual(updated_agreement, test_agreement)

    def test_is_active_type_separation(self):
        """is_active flag is per type"""
        self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_TOS,
                'text': 'Lorem ipsum dolor met sit amet elit',
                'is_active': True,
            },
        )

        self.client.post(
            reverse('misago:admin:users:agreements:new'),
            data={
                'type': Agreement.TYPE_PRIVACY,
                'text': 'Lorem ipsum dolor met sit amet elit',
                'is_active': True,
            },
        )

        active_count = Agreement.objects.filter(is_active=True).count()
        self.assertEqual(active_count, 2)



from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from misago.conf import settings
from misago.core.cache import cache


CACHE_KEY = 'misago_agreements'


class AgreementManager(models.Manager):
    def invalidate_cache(self):
        cache.delete(CACHE_KEY)

    def get_agreements(self):
        agreements = self.get_agreements_from_cache()
        if agreements == 'nada':
            agreements = self.get_agreements_from_db()
            cache.set(CACHE_KEY, agreements)
        return agreements

    def get_agreements_from_cache(self):
        return cache.get(CACHE_KEY, 'nada')

    def get_agreements_from_db(self):
        agreements = {}
        for agreement in Agreement.objects.filter(is_active=True):
            agreements[agreement.type] = {
                'id': agreement.id,
                'title': agreement.get_final_title(),
                'link': agreement.link,
                'text': bool(agreement.text),
            }
        return agreements


class Agreement(models.Model):
    TYPE_TOS = 'terms_of_service'
    TYPE_PRIVACY = 'privacy_policy'
    TYPE_CHOICES = [
        (TYPE_TOS, _('Terms of service')),
        (TYPE_PRIVACY, _('Privacy policy')),
    ]

    type = models.CharField(
        max_length=20,
        default=TYPE_TOS,
        choices=TYPE_CHOICES,
        db_index=True,
    )
    title = models.CharField(max_length=255, null=True, blank=True)
    link = models.URLField(max_length=255, null=True, blank=True)
    text = models.TextField(null=True, blank=True)
    is_active = models.BooleanField(default=False)
    created_on = models.DateTimeField(default=timezone.now)
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        related_name='+',
    )
    created_by_name = models.CharField(max_length=255, null=True, blank=True)
    last_modified_on = models.DateTimeField(null=True, blank=True)
    last_modified_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        blank=True,
        null=True,
        related_name='+',
    )
    last_modified_by_name = models.CharField(max_length=255, null=True, blank=True)

    objects = AgreementManager()

    def get_final_title(self):
        return self.title or self.get_type_display()

    def set_created_by(self, user):
        self.created_by = user
        self.created_by_name = user.username

    def set_last_modified_by(self, user):
        self.last_modified_by = user
        self.last_modified_by_name = user.username


class UserAgreement(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    agreement = models.ForeignKey(Agreement, related_name='accepted_by')
    accepted_on = models.DateTimeField(default=timezone.now)

    class Meta:
        ordering = ["-pk"]


from django.urls import reverse

from .models import Agreement
from .utils import get_parsed_agreement_text, get_required_user_agreement


# fixme: rename this context processor to more suitable name
def legal_links(request):
    agreements = Agreement.objects.get_agreements()

    legal_context = {
        'TERMS_OF_SERVICE_ID': None,
        'TERMS_OF_SERVICE_URL': None,
        'PRIVACY_POLICY_ID': None,
        'PRIVACY_POLICY_URL': None,
        'misago_agreement': None,
    }

    terms_of_service = agreements.get(Agreement.TYPE_TOS)
    if terms_of_service:
        legal_context['TERMS_OF_SERVICE_ID'] = terms_of_service['id']
        if terms_of_service['link']:
            legal_context['TERMS_OF_SERVICE_URL'] = terms_of_service['link']
        elif terms_of_service['text']:
            legal_context['TERMS_OF_SERVICE_URL'] = reverse('misago:terms-of-service')

    privacy_policy = agreements.get(Agreement.TYPE_PRIVACY)
    if privacy_policy:
        legal_context['PRIVACY_POLICY_ID'] = privacy_policy['id']
        if privacy_policy['link']:
            legal_context['PRIVACY_POLICY_URL'] = privacy_policy['link']
        elif privacy_policy['text']:
            legal_context['PRIVACY_POLICY_URL'] = reverse('misago:privacy-policy')

    if legal_context:
        request.frontend_context.update(legal_context)

    required_agreement = get_required_user_agreement(request.user, agreements)
    if required_agreement:
        request.frontend_context['REQUIRED_AGREEMENT_API'] = reverse(
            'misago:api:submit-agreement', kwargs={'pk': required_agreement.pk})

        legal_context['misago_agreement'] = {
            'type': required_agreement.get_type_display(),
            'title': required_agreement.get_final_title(),
            'link': required_agreement.link,
            'text': get_parsed_agreement_text(request, required_agreement)
        }

    return legal_context



from rest_framework import serializers

from misago.users.models import DataDownload


__all__ = ['DataDownloadSerializer']


class DataDownloadSerializer(serializers.ModelSerializer):
    class Meta:
        model = DataDownload
        fields = [
            'id',
            'status',
            'requested_on',
            'expires_on',
            'file',
        ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-06-09 15:23
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('misago_threads', '0009_auto_20180326_0010'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='attachment',
            name='uploader_ip',
        ),
        migrations.RemoveField(
            model_name='poll',
            name='poster_ip',
        ),
        migrations.RemoveField(
            model_name='pollvote',
            name='voter_ip',
        ),
        migrations.RemoveField(
            model_name='post',
            name='poster_ip',
        ),
        migrations.RemoveField(
            model_name='postedit',
            name='editor_ip',
        ),
        migrations.RemoveField(
            model_name='postlike',
            name='liker_ip',
        ),
    ]



from django.test.utils import override_settings

from misago.users.datadownloads import request_user_data_download
from misago.users.testutils import AuthenticatedUserTestCase


class UserRequestDataDownload(AuthenticatedUserTestCase):
    def setUp(self):
        super(UserRequestDataDownload, self).setUp()
        self.link = '/api/users/%s/request-data-download/' % self.user.pk

    def test_request_other_user_download_anonymous(self):
        """requests to api fails if user is anonymous"""
        self.logout_user()

        response = self.client.post(self.link)
        self.assertEqual(response.status_code, 403)
        self.assertEqual(response.json(), {
            'detail': "This action is not available to guests.",
        })

    def test_request_other_user_download(self):
        """requests to api fails if user tries to access other user"""
        other_user = self.get_superuser()
        link = '/api/users/%s/request-data-download/' % other_user.pk

        response = self.client.post(link)
        self.assertEqual(response.status_code, 403)
        self.assertEqual(response.json(), {
            'detail': "You can't request data downloads for other users.",
        })

    @override_settings(MISAGO_ENABLE_DOWNLOAD_OWN_DATA=False)
    def test_request_download_disabled(self):
        """request to api fails if own data downloads are disabled"""
        response = self.client.post(self.link)
        self.assertEqual(response.status_code, 403)
        self.assertEqual(response.json(), {
            'detail': "You can't download your data.",
        })

    def test_request_download_in_progress(self):
        """request to api fails if user has already requested data download"""
        request_user_data_download(self.user)

        response = self.client.post(self.link)
        self.assertEqual(response.status_code, 403)
        self.assertEqual(response.json(), {
            'detail': "You can't have more than one data download request at single time.",
        })

    def test_request_download(self):
        """request to api succeeds"""
        response = self.client.post(self.link)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json(), {
            'detail': "ok",
        })

        self.assertTrue(self.user.datadownload_set.exists())



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-06-03 18:46
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('misago_users', '0011_auto_20180331_2208'),
    ]

    operations = [
        migrations.CreateModel(
            name='AuditTrail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('created_on', models.DateTimeField(db_index=True, default=django.utils.timezone.now)),
                ('ip_address', models.GenericIPAddressField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-pk'],
            },
        ),
    ]



import logging

from django.core.management.base import BaseCommand
from django.utils.translation import ugettext

from misago.conf import settings
from misago.core.mail import mail_user
from misago.core.pgutils import chunk_queryset
from misago.users.datadownloads import prepare_user_data_download
from misago.users.models import DataDownload


logger = logging.getLogger('misago.users.datadownloads')


class Command(BaseCommand):
    help = "Prepares user data downloads."
    leave_locale_alone = True

    def handle(self, *args, **options):
        working_dir = settings.MISAGO_USER_DATA_DOWNLOADS_WORKING_DIR
        if not working_dir:
            self.stdout.write(
                "MISAGO_USER_DATA_DOWNLOADS_WORKING_DIR has to be set in order for "
                "this feature to work.")
            return
        
        downloads_prepared = 0
        queryset = DataDownload.objects.select_related('user')
        queryset = queryset.filter(status=DataDownload.STATUS_PENDING)
        for data_download in chunk_queryset(queryset):
            if prepare_user_data_download(data_download, logger):
                user = data_download.user
                subject = ugettext("%(user)s, your data download is ready") % { 'user': user }
                mail_user(user, subject, 'misago/emails/data_download', context={
                    'data_download': data_download,
                    'expires_in': settings.MISAGO_USER_DATA_DOWNLOADS_EXPIRE_IN_HOURS,
                })

                downloads_prepared += 1

        self.stdout.write("Data downloads prepared: {}".format(downloads_prepared))



from django.test import TestCase, override_settings
from django.urls import reverse

from misago.admin.testutils import AdminTestCase
from misago.admin.views.index import check_misago_address


class AdminIndexViewTests(AdminTestCase):
    def test_view_returns_200(self):
        """admin index view returns 200"""
        response = self.client.get(reverse('misago:admin:index'))

        self.assertContains(response, self.user.username)

    def test_view_contains_address_check(self):
        """admin index view contains address check"""
        response = self.client.get(reverse('misago:admin:index'))

        self.assertContains(response, "MISAGO_ADDRESS")


class RequestMock(object):
    absolute_uri = 'https://misago-project.org/somewhere/'

    def build_absolute_uri(self, location):
        assert location == '/'
        return self.absolute_uri


request = RequestMock()
incorrect_address = 'http://somewhere.com'
correct_address = request.absolute_uri


class AdminIndexAddressCheckTests(TestCase):
    @override_settings(MISAGO_ADDRESS=None)
    def test_address_not_set(self):
        """check handles address not set"""
        result = check_misago_address(request)

        self.assertEqual(result, {
            'is_correct': False,
            'set_address': None,
            'correct_address': request.absolute_uri,
        })

    @override_settings(MISAGO_ADDRESS=incorrect_address)
    def test_address_set_invalid(self):
        """check handles incorrect address"""
        result = check_misago_address(request)

        self.assertEqual(result, {
            'is_correct': False,
            'set_address': incorrect_address,
            'correct_address': request.absolute_uri,
        })

    @override_settings(MISAGO_ADDRESS=correct_address)
    def test_address_set_valid(self):
        """check handles correct address"""
        result = check_misago_address(request)

        self.assertEqual(result, {
            'is_correct': True,
            'set_address': correct_address,
            'correct_address': request.absolute_uri,
        })



# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-08-16 17:29
from __future__ import unicode_literals

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('misago_users', '0014_datadownload'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='agreements',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), default=list, size=None),
        ),
    ]



from django.contrib.auth import get_user_model
from django.core import mail
from django.urls import reverse
from django.utils import six

from misago.acl.models import Role
from misago.admin.testutils import AdminTestCase
from misago.categories.models import Category
from misago.legal.models import Agreement
from misago.legal.utils import save_user_agreement_acceptance
from misago.threads.testutils import post_thread, reply_thread
from misago.users.datadownloads import request_user_data_download
from misago.users.models import Ban, DataDownload, Rank


UserModel = get_user_model()


class UserAdminViewsTests(AdminTestCase):
    AJAX_HEADER = {'HTTP_X_REQUESTED_WITH': 'XMLHttpRequest'}

    def test_link_registered(self):
        """admin index view contains users link"""
        response = self.client.get(reverse('misago:admin:index'))

        self.assertContains(response, reverse('misago:admin:users:accounts:index'))

    def test_list_view(self):
        """users list view returns 200"""
        response = self.client.get(reverse('misago:admin:users:accounts:index'))
        self.assertEqual(response.status_code, 302)

        response = self.client.get(response['location'])
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, self.user.username)

    def test_list_search(self):
        """users list is searchable"""
        response = self.client.get(reverse('misago:admin:users:accounts:index'))
        self.assertEqual(response.status_code, 302)

        link_base = response['location']
        response = self.client.get(link_base)
        self.assertEqual(response.status_code, 200)

        user_a = UserModel.objects.create_user('Tyrael', 't123@test.com', 'pass123')
        user_b = UserModel.objects.create_user('Tyrion', 't321@test.com', 'pass123')
        user_c = UserModel.objects.create_user('Karen', 't432@test.com', 'pass123')

        # Search both
        response = self.client.get('%s&username=tyr' % link_base)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, user_a.username)
        self.assertContains(response, user_b.username)

        # Search tyrion
        response = self.client.get('%s&username=tyrion' % link_base)
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, user_a.username)
        self.assertContains(response, user_b.username)

        # Search tyrael
        response = self.client.get('%s&email=t123@test.com' % link_base)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, user_a.username)
        self.assertNotContains(response, user_b.username)

        # Search disabled
        user_c.is_active = False
        user_c.save()

        response = self.client.get('%s&disabled=1' % link_base)
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, user_a.username)
        self.assertNotContains(response, user_b.username)
        self.assertContains(response, '<del>%s</del>' % user_c.username)

        # Search requested own account delete
        user_c.is_deleting_account = True
        user_c.save()

        response = self.client.get('%s&is_deleting_account=1' % link_base)
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, user_a.username)
        self.assertNotContains(response, user_b.username)
        self.assertContains(response, '<del>%s</del>' % user_c.username)
        
        response = self.client.get('%s&disabled=1' % link_base)
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, user_a.username)
        self.assertNotContains(response, user_b.username)
        self.assertContains(response, '<del>%s</del>' % user_c.username)

    def test_mass_activation(self):
        """users list activates multiple users"""
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
                requires_activation=1,
            )
            user_pks.append(test_user.pk)

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'activate',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 302)

        inactive_qs = UserModel.objects.filter(
            id__in=user_pks,
            requires_activation=1,
        )
        self.assertEqual(inactive_qs.count(), 0)
        self.assertIn("has been activated", mail.outbox[0].subject)

    def test_mass_ban(self):
        """users list bans multiple users"""
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
                requires_activation=1,
            )
            user_pks.append(test_user.pk)

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'ban',
                'selected_items': user_pks,
            }
        )
        self.assertNotContains(response, 'value="ip"')
        self.assertNotContains(response, 'value="ip_first"')
        self.assertNotContains(response, 'value="ip_two"')

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'ban',
                'selected_items': user_pks,
                'ban_type': ['usernames', 'emails', 'domains'],
                'finalize': '',
            },
        )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Ban.objects.count(), 21)

    def test_mass_ban_with_ips(self):
        """users list bans multiple users that also have ips"""
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
                joined_from_ip='73.95.67.27',
                requires_activation=1,
            )
            user_pks.append(test_user.pk)

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'ban',
                'selected_items': user_pks,
            }
        )
        self.assertContains(response, 'value="ip"')
        self.assertContains(response, 'value="ip_first"')
        self.assertContains(response, 'value="ip_two"')

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'ban',
                'selected_items': user_pks,
                'ban_type': ['usernames', 'emails', 'domains', 'ip', 'ip_first', 'ip_two'],
                'finalize': '',
            },
        )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Ban.objects.count(), 24)

    def test_mass_request_data_download(self):
        """users list requests data download for multiple users"""
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
                requires_activation=1,
            )
            user_pks.append(test_user.pk)

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'request_data_download',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(DataDownload.objects.filter(user_id__in=user_pks).count(), len(user_pks))

    def test_mass_request_data_download_avoid_excessive_downloads(self):
        """users list avoids excessive data download requests for multiple users"""
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
                requires_activation=1,
            )
            request_user_data_download(test_user)
            user_pks.append(test_user.pk)

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'v',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(DataDownload.objects.filter(user_id__in=user_pks).count(), len(user_pks))

    def test_mass_delete_accounts_self(self):
        """its impossible to delete oneself"""
        user_pks = [self.user.pk]

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'delete_accounts',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 302)

        response = self.client.get(response['location'])
        self.assertContains(response, "delete yourself")

    def test_mass_delete_accounts_admin(self):
        """its impossible to delete admin account"""
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
            )
            user_pks.append(test_user.pk)

            test_user.is_staff = True
            test_user.save()

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'delete_accounts',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 302)

        response = self.client.get(response['location'])
        self.assertContains(response, "is admin and can")
        self.assertContains(response, "be deleted.")

        self.assertEqual(UserModel.objects.count(), 11)

    def test_mass_delete_accounts_superadmin(self):
        """its impossible to delete superadmin account"""
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
            )
            user_pks.append(test_user.pk)

            test_user.is_superuser = True
            test_user.save()

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'delete_accounts',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 302)

        response = self.client.get(response['location'])
        self.assertContains(response, "is admin and can")
        self.assertContains(response, "be deleted.")

        self.assertEqual(UserModel.objects.count(), 11)

    def test_mass_delete_accounts(self):
        """users list deletes users"""
        # create 10 users to delete
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
                requires_activation=0,
            )
            user_pks.append(test_user.pk)

        # create 10 more users that won't be deleted
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Weebl%s' % i,
                'weebl%s@test.com' % i,
                'pass123',
                requires_activation=0,
            )

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'delete_accounts',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 302)
        self.assertEqual(UserModel.objects.count(), 11)

    def test_mass_delete_all_self(self):
        """its impossible to delete oneself with content"""
        user_pks = [self.user.pk]

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'delete_all',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 302)

        response = self.client.get(response['location'])
        self.assertContains(response, "delete yourself")

    def test_mass_delete_all_admin(self):
        """its impossible to delete admin account and content"""
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
            )
            user_pks.append(test_user.pk)

            test_user.is_staff = True
            test_user.save()

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'delete_all',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 302)

        response = self.client.get(response['location'])
        self.assertContains(response, "is admin and can")
        self.assertContains(response, "be deleted.")

        self.assertEqual(UserModel.objects.count(), 11)

    def test_mass_delete_all_superadmin(self):
        """its impossible to delete superadmin account and content"""
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
            )
            user_pks.append(test_user.pk)

            test_user.is_superuser = True
            test_user.save()

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'delete_all',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 302)

        response = self.client.get(response['location'])
        self.assertContains(response, "is admin and can")
        self.assertContains(response, "be deleted.")

        self.assertEqual(UserModel.objects.count(), 11)

    def test_mass_delete_all(self):
        """users list mass deleting view has no showstoppers"""
        user_pks = []
        for i in range(10):
            test_user = UserModel.objects.create_user(
                'Bob%s' % i,
                'bob%s@test.com' % i,
                'pass123',
                requires_activation=1,
            )
            user_pks.append(test_user.pk)

        response = self.client.post(
            reverse('misago:admin:users:accounts:index'),
            data={
                'action': 'delete_all',
                'selected_items': user_pks,
            }
        )
        self.assertEqual(response.status_code, 200)
         # asser that no user has been deleted, because actuall deleting happens in
         # dedicated views called via ajax from JavaScript
        self.assertEqual(UserModel.objects.count(), 11)

    def test_new_view(self):
        """new user view creates account"""
        response = self.client.get(reverse('misago:admin:users:accounts:new'))
        self.assertEqual(response.status_code, 200)

        default_rank = Rank.objects.get_default()
        authenticated_role = Role.objects.get(special_role='authenticated')

        response = self.client.post(
            reverse('misago:admin:users:accounts:new'),
            data={
                'username': 'Bawww',
                'rank': six.text_type(default_rank.pk),
                'roles': six.text_type(authenticated_role.pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'staff_level': '0',
            }
        )
        self.assertEqual(response.status_code, 302)

        UserModel.objects.get_by_username('Bawww')
        test_user = UserModel.objects.get_by_email('reg@stered.com')

        self.assertTrue(test_user.check_password('pass123'))

    def test_new_view_password_with_whitespaces(self):
        """new user view creates account with whitespaces password"""
        response = self.client.get(reverse('misago:admin:users:accounts:new'))
        self.assertEqual(response.status_code, 200)

        default_rank = Rank.objects.get_default()
        authenticated_role = Role.objects.get(special_role='authenticated')

        response = self.client.post(
            reverse('misago:admin:users:accounts:new'),
            data={
                'username': 'Bawww',
                'rank': six.text_type(default_rank.pk),
                'roles': six.text_type(authenticated_role.pk),
                'email': 'reg@stered.com',
                'new_password': ' pass123 ',
                'staff_level': '0',
            }
        )
        self.assertEqual(response.status_code, 302)

        UserModel.objects.get_by_username('Bawww')
        test_user = UserModel.objects.get_by_email('reg@stered.com')

        self.assertTrue(test_user.check_password(' pass123 '))

    def test_edit_view(self):
        """edit user view changes account"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'newpass123',
                'staff_level': '0',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertTrue(updated_user.check_password('newpass123'))
        self.assertEqual(updated_user.username, 'Bawww')
        self.assertEqual(updated_user.slug, 'bawww')

        UserModel.objects.get_by_username('Bawww')
        UserModel.objects.get_by_email('reg@stered.com')

    def test_edit_dont_change_username(self):
        """
        If username wasn't changed, don't touch user's username, slug or history

        This is regression test for issue #640
        """
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            test_link,
            data={
                'username': 'Bob',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertEqual(updated_user.username, 'Bob')
        self.assertEqual(updated_user.slug, 'bob')
        self.assertEqual(updated_user.namechanges.count(), 0)

    def test_edit_change_password_whitespaces(self):
        """edit user view changes account password to include whitespaces"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': ' newpass123 ',
                'staff_level': '0',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertTrue(updated_user.check_password(' newpass123 '))
        self.assertEqual(updated_user.username, 'Bawww')
        self.assertEqual(updated_user.slug, 'bawww')

        UserModel.objects.get_by_username('Bawww')
        UserModel.objects.get_by_email('reg@stered.com')

    def test_edit_make_admin(self):
        """edit user view allows super admin to make other user admin"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertContains(response, 'id="id_is_staff_1"')
        self.assertContains(response, 'id="id_is_superuser_1"')

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'is_staff': '1',
                'is_superuser': '0',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertTrue(updated_user.is_staff)
        self.assertFalse(updated_user.is_superuser)

    def test_edit_make_superadmin_admin(self):
        """edit user view allows super admin to make other user super admin"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertContains(response, 'id="id_is_staff_1"')
        self.assertContains(response, 'id="id_is_superuser_1"')

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'is_staff': '0',
                'is_superuser': '1',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertFalse(updated_user.is_staff)
        self.assertTrue(updated_user.is_superuser)

    def test_edit_denote_superadmin(self):
        """edit user view allows super admin to denote other super admin"""
        test_user = UserModel.objects.create_user(
            'Bob',
            'bob@test.com',
            'pass123',
            is_staff=True,
            is_superuser=True,
        )

        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertContains(response, 'id="id_is_staff_1"')
        self.assertContains(response, 'id="id_is_superuser_1"')

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'is_staff': '0',
                'is_superuser': '0',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertFalse(updated_user.is_staff)
        self.assertFalse(updated_user.is_superuser)

    def test_edit_cant_make_admin(self):
        """edit user view forbids admins from making other admins"""
        self.user.is_superuser = False
        self.user.save()

        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertNotContains(response, 'id="id_is_staff_1"')
        self.assertNotContains(response, 'id="id_is_superuser_1"')

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'is_staff': '1',
                'is_superuser': '1',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertFalse(updated_user.is_staff)
        self.assertFalse(updated_user.is_superuser)

    def test_edit_disable_user(self):
        """edit user view allows admin to disable non admin"""
        self.user.is_superuser = False
        self.user.save()

        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertContains(response, 'id="id_is_active_1"')
        self.assertContains(response, 'id="id_is_active_staff_message"')

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'is_staff': '0',
                'is_superuser': '0',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
                'is_active': '0',
                'is_active_staff_message': "Disabled in test!"
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertFalse(updated_user.is_active)
        self.assertEqual(updated_user.is_active_staff_message, "Disabled in test!")

    def test_edit_superuser_disable_admin(self):
        """edit user view allows admin to disable non admin"""
        self.user.is_superuser = True
        self.user.save()

        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')

        test_user.is_staff = True
        test_user.save()

        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertContains(response, 'id="id_is_active_1"')
        self.assertContains(response, 'id="id_is_active_staff_message"')

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'is_staff': '1',
                'is_superuser': '0',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
                'is_active': '0',
                'is_active_staff_message': "Disabled in test!"
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertFalse(updated_user.is_active)
        self.assertEqual(updated_user.is_active_staff_message, "Disabled in test!")

    def test_edit_admin_cant_disable_admin(self):
        """edit user view disallows admin to disable admin"""
        self.user.is_superuser = False
        self.user.save()

        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')

        test_user.is_staff = True
        test_user.save()

        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertNotContains(response, 'id="id_is_active_1"')
        self.assertNotContains(response, 'id="id_is_active_staff_message"')

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'is_staff': '1',
                'is_superuser': '0',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
                'is_active': '0',
                'is_active_staff_message': "Disabled in test!"
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertTrue(updated_user.is_active)
        self.assertFalse(updated_user.is_active_staff_message)

    def test_edit_is_deleting_account_cant_reactivate(self):
        """users deleting own accounts can't be reactivated"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_user.mark_for_delete()

        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertNotContains(response, 'id="id_is_active_1"')
        self.assertNotContains(response, 'id="id_is_active_staff_message"')

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'is_staff': '1',
                'is_superuser': '0',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
                'is_active': '1',
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertFalse(updated_user.is_active)
        self.assertTrue(updated_user.is_deleting_account)

    def test_edit_unusable_password(self):
        """admin edit form handles unusable passwords and lets setting new password"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com')
        self.assertFalse(test_user.has_usable_password())

        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertContains(response, 'id="div_id_has_usable_password"')

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': 'pass123',
                'is_staff': '1',
                'is_superuser': '0',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
                'is_active': '1',
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertTrue(updated_user.has_usable_password())

    def test_edit_keep_unusable_password(self):
        """admin edit form handles unusable passwords and lets admin leave them unchanged"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com')
        self.assertFalse(test_user.has_usable_password())

        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.get(test_link)
        self.assertContains(response, 'id="div_id_has_usable_password"')

        response = self.client.post(
            test_link,
            data={
                'username': 'Bawww',
                'rank': six.text_type(test_user.rank_id),
                'roles': six.text_type(test_user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'is_staff': '1',
                'is_superuser': '0',
                'signature': 'Hello world!',
                'is_signature_locked': '1',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': 'Staff message',
                'signature_lock_user_message': 'User message',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
                'is_active': '1',
            }
        )
        self.assertEqual(response.status_code, 302)

        updated_user = UserModel.objects.get(pk=test_user.pk)
        self.assertFalse(updated_user.has_usable_password())

    def test_edit_agreements_list(self):
        """edit view displays list of user's agreements"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:edit', kwargs={
                'pk': test_user.pk,
            }
        )

        agreement = Agreement.objects.create(
            type=Agreement.TYPE_TOS,
            title="Test agreement!",
            text="Lorem ipsum!",
            is_active=True,
        )

        response = self.client.get(test_link)
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, agreement.title)

        save_user_agreement_acceptance(test_user, agreement, commit=True)

        response = self.client.get(test_link)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, agreement.title)

    def test_delete_threads_view_self(self):
        """delete user threads view validates if user deletes self"""
        test_link = reverse(
            'misago:admin:users:accounts:delete-threads', kwargs={
                'pk': self.user.pk,
            }
        )

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:index'))
        self.assertContains(response, "delete yourself");

    def test_delete_threads_view_staff(self):
        """delete user threads view validates if user deletes staff"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_user.is_staff = True
        test_user.save()

        test_link = reverse(
            'misago:admin:users:accounts:delete-threads', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:index'))
        self.assertContains(response, "is admin and");

    def test_delete_threads_view_superuser(self):
        """delete user threads view validates if user deletes superuser"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_user.is_superuser = True
        test_user.save()

        test_link = reverse(
            'misago:admin:users:accounts:delete-threads', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:index'))
        self.assertContains(response, "is admin and");

    def test_delete_threads_view(self):
        """delete user threads view deletes threads"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:delete-threads', kwargs={
                'pk': test_user.pk,
            }
        )

        category = Category.objects.all_categories()[:1][0]
        [post_thread(category, poster=test_user) for _ in range(10)]

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 200)

        response_dict = response.json()
        self.assertEqual(response_dict['deleted_count'], 10)
        self.assertFalse(response_dict['is_completed'])

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 200)

        response_dict = response.json()
        self.assertEqual(response_dict['deleted_count'], 0)
        self.assertTrue(response_dict['is_completed'])

    def test_delete_posts_view_self(self):
        """delete user posts view validates if user deletes self"""
        test_link = reverse(
            'misago:admin:users:accounts:delete-posts', kwargs={
                'pk': self.user.pk,
            }
        )

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:index'))
        self.assertContains(response, "delete yourself");

    def test_delete_posts_view_staff(self):
        """delete user posts view validates if user deletes staff"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_user.is_staff = True
        test_user.save()

        test_link = reverse(
            'misago:admin:users:accounts:delete-posts', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:index'))
        self.assertContains(response, "is admin and");

    def test_delete_posts_view_superuser(self):
        """delete user posts view validates if user deletes superuser"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_user.is_superuser = True
        test_user.save()

        test_link = reverse(
            'misago:admin:users:accounts:delete-posts', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:index'))
        self.assertContains(response, "is admin and");

    def test_delete_posts_view(self):
        """delete user posts view deletes posts"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:delete-posts', kwargs={
                'pk': test_user.pk,
            }
        )

        category = Category.objects.all_categories()[:1][0]
        thread = post_thread(category)
        [reply_thread(thread, poster=test_user) for _ in range(10)]

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 200)

        response_dict = response.json()
        self.assertEqual(response_dict['deleted_count'], 10)
        self.assertFalse(response_dict['is_completed'])

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 200)

        response_dict = response.json()
        self.assertEqual(response_dict['deleted_count'], 0)
        self.assertTrue(response_dict['is_completed'])

    def test_delete_account_view_self(self):
        """delete user account view validates if user deletes self"""
        test_link = reverse(
            'misago:admin:users:accounts:delete-account', kwargs={
                'pk': self.user.pk,
            }
        )

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:index'))
        self.assertContains(response, "delete yourself");

    def test_delete_account_view_staff(self):
        """delete user account view validates if user deletes staff"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_user.is_staff = True
        test_user.save()

        test_link = reverse(
            'misago:admin:users:accounts:delete-account', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:index'))
        self.assertContains(response, "is admin and");

    def test_delete_account_view_superuser(self):
        """delete user account view validates if user deletes superuser"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_user.is_superuser = True
        test_user.save()

        test_link = reverse(
            'misago:admin:users:accounts:delete-account', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 302)

        response = self.client.get(reverse('misago:admin:index'))
        self.assertContains(response, "is admin and");

    def test_delete_account_view(self):
        """delete user account view deletes user account"""
        test_user = UserModel.objects.create_user('Bob', 'bob@test.com', 'pass123')
        test_link = reverse(
            'misago:admin:users:accounts:delete-account', kwargs={
                'pk': test_user.pk,
            }
        )

        response = self.client.post(test_link, **self.AJAX_HEADER)
        self.assertEqual(response.status_code, 200)

        response_dict = response.json()
        self.assertTrue(response_dict['is_completed'])



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-06-24 00:13
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import misago.users.models.datadownload


class Migration(migrations.Migration):

    dependencies = [
        ('misago_users', '0013_auto_20180609_1523'),
    ]

    operations = [
        migrations.CreateModel(
            name='DataDownload',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.PositiveIntegerField(choices=[(0, 'Pending'), (1, 'Processing'), (2, 'Ready'), (3, 'Expired')], db_index=True, default=0)),
                ('requester_name', models.CharField(max_length=255)),
                ('requested_on', models.DateTimeField(default=django.utils.timezone.now)),
                ('expires_on', models.DateTimeField(default=django.utils.timezone.now)),
                ('file', models.FileField(blank=True, max_length=255, null=True, upload_to=misago.users.models.datadownload.get_data_upload_to)),
                ('requester', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-pk'],
            },
        ),
    ]



from hashlib import md5

from django.conf import settings
from django.db import models
from django.utils import timezone
from django.utils.crypto import get_random_string
from django.utils.translation import ugettext_lazy as _


def get_data_upload_to(instance, filename):
    user_id_hexdigest = md5(str(instance.user_id).encode()).hexdigest()
    return 'data-downloads/{}/{}/{}.zip'.format(
        user_id_hexdigest, get_random_string(64), instance.user.slug)


class DataDownload(models.Model):
    STATUS_PENDING = 0
    STATUS_PROCESSING = 1
    STATUS_READY = 2
    STATUS_EXPIRED = 3

    STATUS_CHOICES = [
        (STATUS_PENDING, _("Pending")),
        (STATUS_PROCESSING, _("Processing")),
        (STATUS_READY, _("Ready")),
        (STATUS_EXPIRED, _("Expired")),
    ]

    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT)
    status = models.PositiveIntegerField(
        default=STATUS_PENDING,
        choices=STATUS_CHOICES,
        db_index=True,
    )
    requester = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name='+',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
    )
    requester_name = models.CharField(max_length=255)
    requested_on = models.DateTimeField(default=timezone.now)
    expires_on = models.DateTimeField(default=timezone.now)
    file = models.FileField(upload_to=get_data_upload_to, max_length=255, null=True, blank=True)

    class Meta:
        ordering = ['-pk']

    def delete(self, *args, **kwargs):
        if self.file:
            self.file.delete(save=False)
        super(DataDownload, self).delete(*args, **kwargs)



from django.urls import reverse

from misago.legal.context_processors import legal_links
from misago.legal.models import Agreement
from misago.users.testutils import AuthenticatedUserTestCase


class MockRequest(object):
    def __init__(self, user):
        self.user = user
        self.frontend_context = {}

    def get_host(self):
        return 'testhost.com'


class PrivacyPolicyTests(AuthenticatedUserTestCase):
    def setUp(self):
        super(PrivacyPolicyTests, self).setUp()

        Agreement.objects.invalidate_cache()

    def tearDown(self):
        Agreement.objects.invalidate_cache()

    def test_context_processor_no_policy(self):
        """context processor has no TOS link"""
        context_dict = legal_links(MockRequest(self.user))
        self.assertEqual(context_dict, {
            'TERMS_OF_SERVICE_ID': None,
            'TERMS_OF_SERVICE_URL': None,
            'PRIVACY_POLICY_ID': None,
            'PRIVACY_POLICY_URL': None,
            'misago_agreement': None,
        })

    def test_context_processor_misago_policy(self):
        """context processor has TOS link to Misago view"""
        agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            text='Lorem ipsum',
            is_active=True,
        )

        context_dict = legal_links(MockRequest(self.user))

        self.assertEqual(context_dict, {
            'TERMS_OF_SERVICE_ID': None,
            'TERMS_OF_SERVICE_URL': None,
            'PRIVACY_POLICY_ID': agreement.id,
            'PRIVACY_POLICY_URL': reverse('misago:privacy-policy'),
            'misago_agreement': {
                'type': 'Privacy policy',
                'title': 'Privacy policy',
                'link': None,
                'text': '<p>Lorem ipsum</p>',
            },
        })

    def test_context_processor_remote_policy(self):
        """context processor has TOS link to remote url"""
        agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='http://test.com',
            is_active=True,
        )

        context_dict = legal_links(MockRequest(self.user))

        self.assertEqual(context_dict, {
            'TERMS_OF_SERVICE_ID': None,
            'TERMS_OF_SERVICE_URL': None,
            'PRIVACY_POLICY_ID': agreement.id,
            'PRIVACY_POLICY_URL': 'http://test.com',
            'misago_agreement': {
                'type': 'Privacy policy',
                'title': 'Privacy policy',
                'link': 'http://test.com',
                'text': None,
            },
        })

        # set misago view too
        agreement.text = 'Lorem ipsum'
        agreement.save()
        
        context_dict = legal_links(MockRequest(self.user))

        self.assertEqual(context_dict, {
            'TERMS_OF_SERVICE_ID': None,
            'TERMS_OF_SERVICE_URL': None,
            'PRIVACY_POLICY_ID': agreement.id,
            'PRIVACY_POLICY_URL': 'http://test.com',
            'misago_agreement': {
                'type': 'Privacy policy',
                'title': 'Privacy policy',
                'link': 'http://test.com',
                'text': '<p>Lorem ipsum</p>',
            },
        })


class TermsOfServiceTests(AuthenticatedUserTestCase):
    def setUp(self):
        super(TermsOfServiceTests, self).setUp()
        
        Agreement.objects.invalidate_cache()

    def tearDown(self):
        Agreement.objects.invalidate_cache()

    def test_context_processor_no_tos(self):
        """context processor has no TOS link"""
        context_dict = legal_links(MockRequest(self.user))
        self.assertEqual(context_dict, {
            'TERMS_OF_SERVICE_ID': None,
            'TERMS_OF_SERVICE_URL': None,
            'PRIVACY_POLICY_ID': None,
            'PRIVACY_POLICY_URL': None,
            'misago_agreement': None,
        })

    def test_context_processor_misago_tos(self):
        """context processor has TOS link to Misago view"""
        agreement = Agreement.objects.create(
            type=Agreement.TYPE_TOS,
            text='Lorem ipsum',
            is_active=True,
        )

        context_dict = legal_links(MockRequest(self.user))

        self.assertEqual(context_dict, {
            'TERMS_OF_SERVICE_ID': agreement.id,
            'TERMS_OF_SERVICE_URL': reverse('misago:terms-of-service'),
            'PRIVACY_POLICY_ID': None,
            'PRIVACY_POLICY_URL': None,
            'misago_agreement': {
                'type': 'Terms of service',
                'title': 'Terms of service',
                'link': None,
                'text': '<p>Lorem ipsum</p>',
            }
        })

    def test_context_processor_remote_tos(self):
        """context processor has TOS link to remote url"""
        agreement = Agreement.objects.create(
            type=Agreement.TYPE_TOS,
            link='http://test.com',
            is_active=True,
        )

        context_dict = legal_links(MockRequest(self.user))

        self.assertEqual(context_dict, {
            'TERMS_OF_SERVICE_ID': agreement.id,
            'TERMS_OF_SERVICE_URL': 'http://test.com',
            'PRIVACY_POLICY_ID': None,
            'PRIVACY_POLICY_URL': None,
            'misago_agreement': {
                'type': 'Terms of service',
                'title': 'Terms of service',
                'link': 'http://test.com',
                'text': None,
            }
        })

        # set misago view too
        agreement.text = 'Lorem ipsum'
        agreement.save()

        context_dict = legal_links(MockRequest(self.user))

        self.assertEqual(context_dict, {
            'TERMS_OF_SERVICE_ID': agreement.id,
            'TERMS_OF_SERVICE_URL': 'http://test.com',
            'PRIVACY_POLICY_ID': None,
            'PRIVACY_POLICY_URL': None,
            'misago_agreement': {
                'type': 'Terms of service',
                'title': 'Terms of service',
                'link': 'http://test.com',
                'text': '<p>Lorem ipsum</p>',
            },
        })



from datetime import timedelta

from django.contrib.auth import get_user_model
from django.core.management import call_command
from django.test import TestCase, override_settings
from django.utils import timezone
from django.utils.six import StringIO

from misago.users.management.commands import removeoldips


UserModel = get_user_model()

USER_IP = '31.41.51.65'


class RemoveOldIpsTests(TestCase):
    def test_removeoldips_recent_user(self):
        """command is not removing user's IP if its recent"""
        user = UserModel.objects.create_user('Bob', 'bob@bob.com', joined_from_ip=USER_IP)
        
        out = StringIO()
        call_command(removeoldips.Command(), stdout=out)

        user_joined_from_ip = UserModel.objects.get(pk=user.pk).joined_from_ip
        self.assertEqual(user_joined_from_ip, USER_IP)
    
    def test_removeoldips_old_user(self):
        """command removes user's IP if its old"""
        joined_on_past = timezone.now() - timedelta(days=50)
        user = UserModel.objects.create_user('Bob1', 'bob1@bob.com', joined_from_ip=USER_IP)
        user.joined_on = joined_on_past
        user.save()

        out = StringIO()
        call_command(removeoldips.Command(), stdout=out)

        user_joined_from_ip = UserModel.objects.get(pk=user.pk).joined_from_ip
        self.assertIsNone(user_joined_from_ip)

    @override_settings(MISAGO_IP_STORE_TIME=None)
    def test_not_removing_user_ip(self):
        """command is not removing user's IP if removing is disabled"""
        user = UserModel.objects.create_user('Bob1', 'bob1@bob.com', joined_from_ip=USER_IP)
        
        out = StringIO()
        call_command(removeoldips.Command(), stdout=out)
        command_output = out.getvalue().splitlines()[0].strip()
        
        self.assertEqual(command_output, "Old IP removal is disabled.")
        
        user_joined_from_ip = UserModel.objects.get(pk=user.pk).joined_from_ip
        self.assertEqual(user_joined_from_ip, USER_IP)




# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-06-09 15:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('misago_users', '0012_audittrail'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='online',
            name='current_ip',
        ),
        migrations.RemoveField(
            model_name='user',
            name='last_ip',
        ),
        migrations.AlterField(
            model_name='user',
            name='joined_from_ip',
            field=models.GenericIPAddressField(blank=True, null=True),
        ),
    ]



from django.dispatch import receiver

from misago.users.signals import anonymize_user_data, username_changed

from .models import Agreement


@receiver([anonymize_user_data, username_changed])
def update_usernames(sender, **kwargs):
    Agreement.objects.filter(created_by=sender).update(
        created_by_name=sender.username,
    )

    Agreement.objects.filter(last_modified_by=sender).update(
        last_modified_by_name=sender.username,
    )


# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-08-16 14:22
from __future__ import unicode_literals

from django.db import migrations

from misago.conf.migrationutils import migrate_settings_group
from misago.legal.models import Agreement as MisagoAgreement


_ = lambda s: s


LEGAL_SETTINGS = [
    'terms_of_service_title',
    'terms_of_service_link',
    'terms_of_service',
    'privacy_policy_title',
    'privacy_policy_link',
    'privacy_policy',
]


def create_legal_settings_group(apps, schema_editor):
    Agreement = apps.get_model('misago_legal', 'Agreement')
    Setting = apps.get_model('misago_conf', 'Setting')
    
    legal_conf = {}
    for setting in Setting.objects.filter(setting__in=LEGAL_SETTINGS):
        legal_conf[setting.setting] = setting.dry_value

    if legal_conf['terms_of_service'] or legal_conf['terms_of_service_link']:
        Agreement.objects.create(
            type=MisagoAgreement.TYPE_TOS,
            title=legal_conf['terms_of_service_title'],
            link=legal_conf['terms_of_service_link'],
            text=legal_conf['terms_of_service'],
            is_active=True,
        )

    if legal_conf['privacy_policy'] or legal_conf['privacy_policy_link']:
        Agreement.objects.create(
            type=MisagoAgreement.TYPE_PRIVACY,
            title=legal_conf['privacy_policy_title'],
            link=legal_conf['privacy_policy_link'],
            text=legal_conf['privacy_policy'],
            is_active=True,
        )

    MisagoAgreement.objects.invalidate_cache()


def delete_deprecated_settings(apps, schema_editor):
    migrate_settings_group(
        apps, {
            'key': 'legal',
            'name': _("Legal information"),
            'description': _("Those settings allow you to set additional legal information for your forum."),
            'settings': [
                {
                    'setting': 'forum_footnote',
                    'name': _("Footnote"),
                    'description': _("Short message displayed in forum footer."),
                    'legend': _("Forum footer"),
                    'field_extra': {
                        'max_length': 300,
                    },
                    'is_public': True,
                },
            ],
        }
    )


class Migration(migrations.Migration):

    dependencies = [
        ('misago_legal', '0002_agreement_useragreement'),
    ]

    operations = [
        migrations.RunPython(create_legal_settings_group),
        migrations.RunPython(delete_deprecated_settings),
    ]



from datetime import timedelta

from django.contrib.auth import get_user_model
from django.core.management import call_command
from django.test import TestCase, override_settings
from django.utils import timezone
from django.utils.six import StringIO

from misago.users.management.commands import deleteinactiveusers


UserModel = get_user_model()


class DeleteInactiveUsersTests(TestCase):
    def setUp(self):
        self.user = UserModel.objects.create_user('Bob', 'bob@bob.com', 'pass123')

    @override_settings(MISAGO_DELETE_NEW_INACTIVE_USERS_OLDER_THAN_DAYS=2)
    def test_delete_user_activation_user(self):
        """deletes user that didn't activate their account within required time"""
        self.user.joined_on = timezone.now() - timedelta(days=2)
        self.user.requires_activation = UserModel.ACTIVATION_USER
        self.user.save()

        out = StringIO()
        call_command(deleteinactiveusers.Command(), stdout=out)
        command_output = out.getvalue().splitlines()[0].strip()

        self.assertEqual(command_output, "Deleted users: 1")

        with self.assertRaises(UserModel.DoesNotExist):
            UserModel.objects.get(pk=self.user.pk)

    @override_settings(MISAGO_DELETE_NEW_INACTIVE_USERS_OLDER_THAN_DAYS=2)
    def test_delete_user_activation_admin(self):
        """deletes user that wasn't activated by admin within required time"""
        self.user.joined_on = timezone.now() - timedelta(days=2)
        self.user.requires_activation = UserModel.ACTIVATION_ADMIN
        self.user.save()

        out = StringIO()
        call_command(deleteinactiveusers.Command(), stdout=out)
        command_output = out.getvalue().splitlines()[0].strip()

        self.assertEqual(command_output, "Deleted users: 1")

        with self.assertRaises(UserModel.DoesNotExist):
            UserModel.objects.get(pk=self.user.pk)

    @override_settings(MISAGO_DELETE_NEW_INACTIVE_USERS_OLDER_THAN_DAYS=2)
    def test_skip_new_user_activation_user(self):
        """skips inactive user that is too new"""
        self.user.joined_on = timezone.now() - timedelta(days=1)
        self.user.requires_activation = UserModel.ACTIVATION_USER
        self.user.save()

        out = StringIO()
        call_command(deleteinactiveusers.Command(), stdout=out)
        command_output = out.getvalue().splitlines()[0].strip()

        self.assertEqual(command_output, "Deleted users: 0")

        UserModel.objects.get(pk=self.user.pk)

    @override_settings(MISAGO_DELETE_NEW_INACTIVE_USERS_OLDER_THAN_DAYS=2)
    def test_skip_new_user_activation_admin(self):
        """skips admin-activated user that is too new"""
        self.user.joined_on = timezone.now() - timedelta(days=1)
        self.user.requires_activation = UserModel.ACTIVATION_ADMIN
        self.user.save()

        out = StringIO()
        call_command(deleteinactiveusers.Command(), stdout=out)
        command_output = out.getvalue().splitlines()[0].strip()

        self.assertEqual(command_output, "Deleted users: 0")

        UserModel.objects.get(pk=self.user.pk)

    @override_settings(MISAGO_DELETE_NEW_INACTIVE_USERS_OLDER_THAN_DAYS=2)
    def test_skip_active_user(self):
        """skips active user"""
        self.user.joined_on = timezone.now() - timedelta(days=1)
        self.user.save()

        out = StringIO()
        call_command(deleteinactiveusers.Command(), stdout=out)
        command_output = out.getvalue().splitlines()[0].strip()

        self.assertEqual(command_output, "Deleted users: 0")

        UserModel.objects.get(pk=self.user.pk)

    @override_settings(MISAGO_DELETE_NEW_INACTIVE_USERS_OLDER_THAN_DAYS=0)
    def test_delete_inactive_is_disabled(self):
        """skips active user"""
        self.user.joined_on = timezone.now() - timedelta(days=1)
        self.user.requires_activation = UserModel.ACTIVATION_ADMIN
        self.user.save()

        out = StringIO()
        call_command(deleteinactiveusers.Command(), stdout=out)
        command_output = out.getvalue().splitlines()[0].strip()

        self.assertEqual(
            command_output, "Automatic deletion of inactive users is currently disabled.")

        UserModel.objects.get(pk=self.user.pk)



from django.contrib import messages
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from misago.admin.views import generic

from misago.legal.forms import AgreementForm, SearchAgreementsForm
from misago.legal.models import Agreement
from misago.legal.utils import set_agreement_as_active


class AgreementAdmin(generic.AdminBaseMixin):
    root_link = 'misago:admin:users:agreements:index'
    model = Agreement
    form = AgreementForm
    templates_dir = 'misago/admin/agreements'
    message_404 = _("Requested agreement does not exist.")

    def handle_form(self, form, request, target):
        form.save()
        
        if self.message_submit:
            messages.success(request, self.message_submit % {'title': target.get_final_title()})


class AgreementsList(AgreementAdmin, generic.ListView):
    items_per_page = 30
    ordering = [
        ('-id', _("From newest")),
        ('id', _("From oldest")),
    ]
    search_form = SearchAgreementsForm
    selection_label = _('With agreements: 0')
    empty_selection_label = _('Select agreements')
    mass_actions = ({
        'action': 'delete',
        'icon': 'fa fa-times',
        'name': _('Delete agreements'),
        'confirmation': _('Are you sure you want to delete those agreements?')
    }, )

    def get_queryset(self):
        qs = super(AgreementsList, self).get_queryset()
        return qs.select_related()

    def action_delete(self, request, items):
        items.delete()
        Agreement.objects.invalidate_cache()
        messages.success(request, _("Selected agreements have been deleted."))


class NewAgreement(AgreementAdmin, generic.ModelFormView):
    message_submit = _('New agreement "%(title)s" has been saved.')
    
    def handle_form(self, form, request, target):
        super(NewAgreement, self).handle_form(form, request, target)

        form.instance.set_created_by(request.user)
        form.instance.save()


class EditAgreement(AgreementAdmin, generic.ModelFormView):
    message_submit = _('Agreement "%(title)s" has been edited.')

    def handle_form(self, form, request, target):
        super(EditAgreement, self).handle_form(form, request, target)

        form.instance.last_modified_on = timezone.now()
        form.instance.set_last_modified_by(request.user)
        form.instance.save()


class DeleteAgreement(AgreementAdmin, generic.ButtonView):
    def button_action(self, request, target):
        target.delete()
        Agreement.objects.invalidate_cache()
        message = _('Agreement "%(title)s" has been deleted.')
        messages.success(request, message % {'title': target.get_final_title()})


class SetAgreementAsActive(AgreementAdmin, generic.ButtonView):
    def button_action(self, request, target):
        set_agreement_as_active(target, commit=True)

        message = _('Agreement "%(title)s" has been set as active for type "%(type)s".')
        targets_names = {'title': target.get_final_title(), 'type': target.get_type_display()}
        messages.success(request, message % targets_names)


from django.contrib.auth import get_user_model
from django.urls import reverse
from django.utils import six

from misago.admin.testutils import AdminTestCase


UserModel = get_user_model()


class GenderProfileFieldTests(AdminTestCase):
    def setUp(self):
        super(GenderProfileFieldTests, self).setUp()

        self.test_link = reverse(
            'misago:admin:users:accounts:edit',
            kwargs={
                'pk': self.user.pk,
            },
        )

    def test_field_displays_in_admin(self):
        """field displays in admin"""
        response = self.client.get(self.test_link)
        self.assertContains(response, 'name="gender"')

    def test_admin_clears_field(self):
        """admin form allows admins to clear field"""
        self.user.profile_fields['gender'] = 'female'
        self.user.save()

        self.reload_user()
        self.assertEqual(self.user.profile_fields['gender'], 'female')

        response = self.client.post(
            self.test_link,
            data={
                'username': 'Edited',
                'rank': six.text_type(self.user.rank_id),
                'roles': six.text_type(self.user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'new_password': '',
                'signature': '',
                'is_signature_locked': '0',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': '',
                'signature_lock_user_message': '',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
            }
        )
        self.assertEqual(response.status_code, 302)

        self.reload_user()
        self.assertEqual(self.user.profile_fields['gender'], '')

    def test_admin_validates_field(self):
        """admin form allows admins to edit field"""
        response = self.client.post(
            self.test_link,
            data={
                'username': 'Edited',
                'rank': six.text_type(self.user.rank_id),
                'roles': six.text_type(self.user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'gender': 'attackcopter',
                'new_password': '',
                'signature': '',
                'is_signature_locked': '0',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': '',
                'signature_lock_user_message': '',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
            }
        )

        self.assertContains(response, 'attackcopter is not one of the available choices.')

    def test_admin_edits_field(self):
        """admin form allows admins to edit field"""
        response = self.client.post(
            self.test_link,
            data={
                'username': 'Edited',
                'rank': six.text_type(self.user.rank_id),
                'roles': six.text_type(self.user.roles.all()[0].pk),
                'email': 'reg@stered.com',
                'gender': 'female',
                'new_password': '',
                'signature': '',
                'is_signature_locked': '0',
                'is_hiding_presence': '0',
                'limits_private_thread_invites_to': '0',
                'signature_lock_staff_message': '',
                'signature_lock_user_message': '',
                'subscribe_to_started_threads': '2',
                'subscribe_to_replied_threads': '2',
            }
        )
        self.assertEqual(response.status_code, 302)

        self.reload_user()
        self.assertEqual(self.user.profile_fields['gender'], 'female')

    def test_admin_search_field(self):
        """admin users search searches this field"""
        test_link = reverse('misago:admin:users:accounts:index')

        response = self.client.get('{}?redirected=1&profilefields=female'.format(test_link))
        self.assertContains(response, "No users matching search criteria have been found.")

        # search by value
        self.user.profile_fields['gender'] = 'female'
        self.user.save()

        response = self.client.get('{}?redirected=1&profilefields=female'.format(test_link))
        self.assertNotContains(response, "No users matching search criteria have been found.")

        # search by choice name
        self.user.profile_fields['gender'] = 'secret'
        self.user.save()

        response = self.client.get('{}?redirected=1&profilefields=telling'.format(test_link))
        self.assertNotContains(response, "No users matching search criteria have been found.")

    def test_field_display(self):
        """field displays on user profile when filled in"""
        test_link = reverse(
            'misago:user-details',
            kwargs={
                'pk': self.user.pk,
                'slug': self.user.slug,
            },
        )

        response = self.client.get(test_link)
        self.assertNotContains(response, 'Gender')

        self.user.profile_fields['gender'] = 'secret'
        self.user.save()

        response = self.client.get(test_link)
        self.assertContains(response, 'Gender')
        self.assertContains(response, 'Not telling')

    def test_field_outdated_hidden(self):
        """field with outdated value is hidden"""
        test_link = reverse(
            'misago:user-details',
            kwargs={
                'pk': self.user.pk,
                'slug': self.user.slug,
            },
        )

        response = self.client.get(test_link)
        self.assertNotContains(response, 'Gender')

        self.user.profile_fields['gender'] = 'not valid'
        self.user.save()

        response = self.client.get(test_link)
        self.assertNotContains(response, 'Gender')

    def test_field_display_json(self):
        """field is included in display json"""
        test_link = reverse('misago:api:user-details', kwargs={'pk': self.user.pk})

        response = self.client.get(test_link)
        self.assertEqual(
            response.json()['groups'],
            [
                {
                    'name': 'IP address',
                    'fields': [
                        {
                            'fieldname': 'join_ip',
                            'name': 'Join IP',
                            'text': '127.0.0.1',
                        },
                    ],
                },
            ]
        )

        self.user.profile_fields['gender'] = 'male'
        self.user.save()

        response = self.client.get(test_link)
        self.assertEqual(
            response.json()['groups'],
            [
                {
                    'name': 'Personal',
                    'fields': [
                        {
                            'fieldname': 'gender',
                            'name': 'Gender',
                            'text': 'Male',
                        }
                    ],
                },
                {
                    'name': 'IP address',
                    'fields': [
                        {
                            'fieldname': 'join_ip',
                            'name': 'Join IP',
                            'text': '127.0.0.1',
                        },
                    ],
                },
            ]
        )

    def test_field_outdated_hidden_json(self):
        """field with outdated value is removed in display json"""
        test_link = reverse('misago:api:user-details', kwargs={'pk': self.user.pk})

        response = self.client.get(test_link)
        self.assertEqual(
            response.json()['groups'],
            [
                {
                    'name': 'IP address',
                    'fields': [
                        {
                            'fieldname': 'join_ip',
                            'name': 'Join IP',
                            'text': '127.0.0.1',
                        },
                    ],
                },
            ]
        )

        self.user.profile_fields['gender'] = 'invalid'
        self.user.save()

        response = self.client.get(test_link)
        self.assertEqual(
            response.json()['groups'],
            [
                {
                    'name': 'IP address',
                    'fields': [
                        {
                            'fieldname': 'join_ip',
                            'name': 'Join IP',
                            'text': '127.0.0.1',
                        },
                    ],
                },
            ]
        )

    def test_api_returns_field_json(self):
        """field json is returned from API"""
        test_link = reverse('misago:api:user-edit-details', kwargs={'pk': self.user.pk})

        response = self.client.get(test_link)

        found_field = None
        for group in response.json():
            for field in group['fields']:
                if field['fieldname'] == 'gender':
                    found_field = field

        self.assertEqual(found_field, {
            'fieldname': 'gender',
            'label': 'Gender',
            'help_text': None,
            'input': {
                'type': 'select',
                'choices': [
                    {'label': 'Not specified', 'value': ''},
                    {'label': 'Not telling', 'value': 'secret'},
                    {'label': 'Female', 'value': 'female'},
                    {'label': 'Male', 'value': 'male'},
                ],
            },
            'initial': '',
        })

    def test_api_clears_field(self):
        """field can be cleared via api"""
        test_link = reverse('misago:api:user-edit-details', kwargs={'pk': self.user.pk})

        self.user.profile_fields['gender'] = 'secret'
        self.user.save()

        self.reload_user()
        self.assertEqual(self.user.profile_fields['gender'], 'secret')

        response = self.client.post(test_link, data={})
        self.assertEqual(response.status_code, 200)

        self.reload_user()
        self.assertEqual(self.user.profile_fields['gender'], '')

    def test_api_validates_field(self):
        """field can be edited via api"""
        test_link = reverse('misago:api:user-edit-details', kwargs={'pk': self.user.pk})

        response = self.client.post(test_link, data={'gender': 'attackhelicopter'})
        self.assertContains(response, "attackhelicopter is not one of the available choices.", status_code=400)

    def test_api_edits_field(self):
        """field can be edited via api"""
        test_link = reverse('misago:api:user-edit-details', kwargs={'pk': self.user.pk})

        response = self.client.post(test_link, data={'gender': 'female'})
        self.assertEqual(response.status_code, 200)

        self.reload_user()
        self.assertEqual(self.user.profile_fields['gender'], 'female')



import requests
from requests.exceptions import RequestException

from django.contrib.auth import get_user_model
from django.http import Http404, JsonResponse
from django.utils.translation import ugettext as _

from misago import __version__
from misago.conf import settings
from misago.core.cache import cache
from misago.threads.models import Post, Thread

from . import render


VERSION_CHECK_CACHE_KEY = "misago_version_check"

UserModel = get_user_model()


def admin_index(request):
    inactive_users_queryset = UserModel.objects.exclude(
        requires_activation=UserModel.ACTIVATION_NONE,
    )

    db_stats = {
        'threads': Thread.objects.count(),
        'posts': Post.objects.count(),
        'users': UserModel.objects.count(),
        'inactive_users': inactive_users_queryset.count()
    }

    return render(
        request, 'misago/admin/index.html', {
            'db_stats': db_stats,
            'address_check': check_misago_address(request),
            'version_check': cache.get(VERSION_CHECK_CACHE_KEY),
        }
    )


def check_misago_address(request):
    set_address = settings.MISAGO_ADDRESS
    correct_address = request.build_absolute_uri('/')

    return {
        'is_correct': set_address == correct_address,
        'set_address': set_address,
        'correct_address': correct_address,
    }


def check_version(request):
    if request.method != "POST":
        raise Http404()

    version = cache.get(VERSION_CHECK_CACHE_KEY, 'nada')

    if version == 'nada':
        try:
            api_url = 'https://pypi.org/pypi/Misago/json'
            r = requests.get(api_url)
            r.raise_for_status()

            latest_version = r.json()['info']['version']

            if latest_version == __version__:
                version = {
                    'is_error': False,
                    'message': _("Up to date! (%(current)s)") % {
                        'current': __version__,
                    },
                }
            else:
                version = {
                    'is_error': True,
                    'message': _("Outdated: %(current)s! (latest: %(latest)s)") % {
                        'latest': latest_version,
                        'current': __version__,
                    },
                }

            cache.set(VERSION_CHECK_CACHE_KEY, version, 180)
        except (RequestException, IndexError, KeyError, ValueError) as e:
            version = {
                'is_error': True,
                'message': _("Failed to connect to pypi.org API. Try again later."),
            }

    return JsonResponse(version)



from django.apps import AppConfig


class MisagoLegalConfig(AppConfig):
    name = 'misago.legal'
    label = 'misago_legal'
    verbose_name = "Misago Legal"

    def ready(self):
        from . import signals as _



from django.utils.translation import ugettext as _

from misago.conf import settings
from misago.core.mail import mail_user
from misago.legal.models import Agreement
from misago.legal.utils import save_user_agreement_acceptance
from misago.users.tokens import make_activation_token


def send_welcome_email(request, user):
    mail_subject = _("Welcome on %(forum_name)s forums!")
    mail_subject = mail_subject % {'forum_name': settings.forum_name}

    if not user.requires_activation:
        mail_user(user, mail_subject, 'misago/emails/register/complete')
        return

    activation_token = make_activation_token(user)

    activation_by_admin = user.requires_activation_by_admin
    activation_by_user = user.requires_activation_by_user

    mail_user(
        user,
        mail_subject,
        'misago/emails/register/inactive',
        context={
            'activation_token': activation_token,
            'activation_by_admin': activation_by_admin,
            'activation_by_user': activation_by_user,
        }
    )


def save_user_agreements(user, form):
    if not form.agreements:
        return

    for field_name in form.agreements.keys():
        agreement_id = form.cleaned_data[field_name]
        agreement = Agreement.objects.get(id=agreement_id)
        save_user_agreement_acceptance(user, agreement)

    user.save(update_fields=['agreements'])


def get_registration_result_json(user):
    activation_method = 'active'
    if user.requires_activation_by_admin:
        activation_method = 'admin'
    elif user.requires_activation_by_user:
        activation_method = 'user'

    return {
        'activation': activation_method,
        'email': user.email,
        'username': user.username,
    }


# -*- coding: utf-8 -*-
# Generated by Django 1.11.15 on 2018-08-15 20:58
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('misago_legal', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Agreement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(choices=[('terms_of_service', 'Terms of service'), ('privacy_policy', 'Privacy policy')], db_index=True, default='terms_of_service', max_length=20)),
                ('title', models.CharField(blank=True, max_length=255, null=True)),
                ('link', models.URLField(blank=True, max_length=255, null=True)),
                ('text', models.TextField(blank=True, null=True)),
                ('is_active', models.BooleanField(default=False)),
                ('created_on', models.DateTimeField(default=django.utils.timezone.now)),
                ('created_by_name', models.CharField(blank=True, max_length=255, null=True)),
                ('last_modified_on', models.DateTimeField(blank=True, null=True)),
                ('last_modified_by_name', models.CharField(blank=True, max_length=255, null=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('last_modified_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='UserAgreement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('accepted_on', models.DateTimeField(default=django.utils.timezone.now)),
                ('agreement', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='accepted_by', to='misago_legal.Agreement')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-pk'],
            },
        ),
    ]



from django.test import TestCase

from misago.legal.models import Agreement, UserAgreement
from misago.legal.utils import (
    get_parsed_agreement_text, get_required_user_agreement, save_user_agreement_acceptance,
    set_agreement_as_active
)
from misago.users.testutils import UserTestCase


class MockRequest(object):
    def __init__(self, user=None):
        self.user = user
        self.frontend_context = {}

    def get_host(self):
        return 'testhost.com'


class GetParsedAgreementTextTests(TestCase):
    def test_agreement_no_text(self):
        agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='https://somewhre.com',
            is_active=True,
        )

        result = get_parsed_agreement_text(MockRequest(), agreement)
        self.assertIsNone(result)

    def test_agreement_link_and_text(self):
        agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='https://somewhre.com',
            text='Lorem ipsum',
            is_active=True,
        )

        result = get_parsed_agreement_text(MockRequest(), agreement)
        self.assertEqual(result, '<p>Lorem ipsum</p>')

    def test_agreement_text(self):
        agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            text='Lorem ipsum',
            is_active=True,
        )

        result = get_parsed_agreement_text(MockRequest(), agreement)
        self.assertEqual(result, '<p>Lorem ipsum</p>')


class GetRequiredUserAgreementTests(UserTestCase):
    def setUp(self):
        self.agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='https://somewhre.com',
            text='Lorem ipsum',
            is_active=True,
        )

        self.agreements = Agreement.objects.get_agreements_from_db()

    def test_anonymous_user(self):
        anonymous_user = self.get_anonymous_user()
        result = get_required_user_agreement(anonymous_user, self.agreements)
        self.assertIsNone(result)

    def test_authenticated_user_no_agreements(self):
        authenticated_user = self.get_authenticated_user()
        result = get_required_user_agreement(authenticated_user, {})
        self.assertIsNone(result)

    def test_authenticated_user(self):
        authenticated_user = self.get_authenticated_user()
        result = get_required_user_agreement(authenticated_user, self.agreements)
        self.assertEqual(result, self.agreement)

    def test_authenticated_user_with_agreement(self):
        authenticated_user = self.get_authenticated_user()
        authenticated_user.agreements.append(self.agreement.pk)

        result = get_required_user_agreement(authenticated_user, self.agreements)
        self.assertIsNone(result)

    def test_prioritize_terms_of_service(self):
        terms_of_service = Agreement.objects.create(
            type=Agreement.TYPE_TOS,
            link='https://somewhre.com',
            text='Lorem ipsum',
            is_active=True,
        )

        agreements = Agreement.objects.get_agreements_from_db()
        agreements_in_wrong_order = {
            Agreement.TYPE_PRIVACY: agreements[Agreement.TYPE_PRIVACY],
            Agreement.TYPE_TOS: agreements[Agreement.TYPE_TOS],
        }

        authenticated_user = self.get_authenticated_user()
        result = get_required_user_agreement(authenticated_user, agreements_in_wrong_order)
        self.assertEqual(result, terms_of_service)


class SaveUserAgreementAcceptance(UserTestCase):
    def test_no_commit(self):
        user = self.get_authenticated_user()

        agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='https://somewhre.com',
            text='Lorem ipsum',
        )

        save_user_agreement_acceptance(user, agreement)
        self.assertEqual(user.agreements, [agreement.id])

        user.refresh_from_db()
        self.assertEqual(user.agreements, [])

        UserAgreement.objects.get(user=user, agreement=agreement)
        self.assertEqual(UserAgreement.objects.count(), 1)

    def test_commit(self):
        user = self.get_authenticated_user()

        agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='https://somewhre.com',
            text='Lorem ipsum',
        )

        save_user_agreement_acceptance(user, agreement, commit=True)
        self.assertEqual(user.agreements, [agreement.id])

        user.refresh_from_db()
        self.assertEqual(user.agreements, [agreement.id])

        UserAgreement.objects.get(user=user, agreement=agreement)
        self.assertEqual(UserAgreement.objects.count(), 1)


class SetAgreementAsActiveTests(TestCase):
     def test_inactive_agreement(self):
        agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='https://somewhre.com',
            text='Lorem ipsum',
        )

        set_agreement_as_active(agreement)
        self.assertTrue(agreement.is_active)

        agreement.refresh_from_db()
        self.assertFalse(agreement.is_active)

     def test_inactive_agreement_commit(self):
        agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='https://somewhre.com',
            text='Lorem ipsum',
        )

        set_agreement_as_active(agreement, commit=True)
        self.assertTrue(agreement.is_active)

        agreement.refresh_from_db()
        self.assertTrue(agreement.is_active)
        
     def test_change_active_agreement(self):
        old_agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='https://somewhre.com',
            text='Lorem ipsum',
            is_active=True,
        )

        new_agreement = Agreement.objects.create(
            type=Agreement.TYPE_PRIVACY,
            link='https://somewhre.com',
            text='Lorem ipsum',
        )

        other_type_agreement = Agreement.objects.create(
            type=Agreement.TYPE_TOS,
            link='https://somewhre.com',
            text='Lorem ipsum',
            is_active=True,
        )

        set_agreement_as_active(new_agreement, commit=True)

        old_agreement.refresh_from_db()
        new_agreement.refresh_from_db()
        other_type_agreement.refresh_from_db()

        self.assertFalse(old_agreement.is_active)
        self.assertTrue(new_agreement.is_active)
        self.assertTrue(other_type_agreement.is_active)



from datetime import timedelta

from django.contrib.auth import get_user_model
from django.utils import timezone

from misago.users.audittrail import create_audit_trail, create_user_audit_trail
from misago.users.models import AuditTrail
from misago.users.signals import remove_old_ips
from misago.users.testutils import UserTestCase


UserModel = get_user_model()

USER_IP = '13.41.51.41'


class MockRequest(object):
    user_ip = USER_IP

    def __init__(self, user):
        self.user = user


class CreateAuditTrailTests(UserTestCase):
    def setUp(self):
        super(CreateAuditTrailTests, self).setUp()

        self.obj = UserModel.objects.create_user('BobBoberson', 'bob@example.com')

    def test_create_audit_require_model(self):
        """create_audit_trail requires model instance"""
        anonymous_user = self.get_anonymous_user()
        request = MockRequest(anonymous_user)
        with self.assertRaises(ValueError):
            create_audit_trail(request, anonymous_user)
        self.assertEqual(AuditTrail.objects.count(), 0)

    def test_create_audit_trail_anonymous_user(self):
        """create_audit_trail doesn't record anonymous users"""
        user = self.get_anonymous_user()
        request = MockRequest(user)
        create_audit_trail(request, self.obj)
        self.assertEqual(AuditTrail.objects.count(), 0)

    def test_create_audit_trail(self):
        """create_audit_trail creates new db record"""
        user = self.get_authenticated_user()
        request = MockRequest(user)
        create_audit_trail(request, self.obj)
        self.assertEqual(AuditTrail.objects.count(), 1)

        audit_trail = user.audittrail_set.all()[0]
        self.assertEqual(audit_trail.user, user)
        self.assertEqual(audit_trail.ip_address, request.user_ip)
        self.assertEqual(audit_trail.content_object, self.obj)

    def test_delete_user_remove_audit_trail(self):
        """audit trail is deleted together with user it belongs to"""
        user = self.get_authenticated_user()
        request = MockRequest(user)
        create_audit_trail(request, self.obj)
        self.assertEqual(AuditTrail.objects.count(), 1)

        user.delete()
        self.assertEqual(AuditTrail.objects.count(), 0)

    def test_delete_obj_keep_audit_trail(self):
        """audit trail is kept after with obj it points at is deleted"""
        user = self.get_authenticated_user()
        request = MockRequest(user)
        create_audit_trail(request, self.obj)
        self.assertEqual(AuditTrail.objects.count(), 1)

        self.obj.delete()
        self.assertEqual(AuditTrail.objects.count(), 1)

        audit_trail = user.audittrail_set.all()[0]
        self.assertEqual(audit_trail.user, user)
        self.assertEqual(audit_trail.ip_address, request.user_ip)
        self.assertIsNone(audit_trail.content_object)

    def test_delete_audit_trail(self):
        """audit trail deletion leaves other data untouched"""
        user = self.get_authenticated_user()
        request = MockRequest(user)
        create_audit_trail(request, self.obj)
        self.assertEqual(AuditTrail.objects.count(), 1)

        audit_trail = user.audittrail_set.all()[0]
        audit_trail.delete()
        
        UserModel.objects.get(id=user.id)
        UserModel.objects.get(id=self.obj.id)


class CreateUserAuditTrailTests(UserTestCase):
    def setUp(self):
        super(CreateUserAuditTrailTests, self).setUp()

        self.obj = UserModel.objects.create_user('BobBoberson', 'bob@example.com')

    def test_create_user_audit_require_model(self):
        """create_user_audit_trail requires model instance"""
        anonymous_user = self.get_anonymous_user()
        with self.assertRaises(ValueError):
            create_user_audit_trail(anonymous_user, USER_IP, anonymous_user)
        self.assertEqual(AuditTrail.objects.count(), 0)

    def test_create_user_audit_trail_anonymous_user(self):
        """create_user_audit_trail doesn't record anonymous users"""
        user = self.get_anonymous_user()
        create_user_audit_trail(user, USER_IP, self.obj)
        self.assertEqual(AuditTrail.objects.count(), 0)

    def test_create_user_audit_trail(self):
        """create_user_audit_trail creates new db record"""
        user = self.get_authenticated_user()
        create_user_audit_trail(user, USER_IP, self.obj)
        self.assertEqual(AuditTrail.objects.count(), 1)

        audit_trail = user.audittrail_set.all()[0]
        self.assertEqual(audit_trail.user, user)
        self.assertEqual(audit_trail.ip_address, USER_IP)
        self.assertEqual(audit_trail.content_object, self.obj)

    def test_delete_user_remove_audit_trail(self):
        """audit trail is deleted together with user it belongs to"""
        user = self.get_authenticated_user()
        create_user_audit_trail(user, USER_IP, self.obj)
        self.assertEqual(AuditTrail.objects.count(), 1)

        user.delete()
        self.assertEqual(AuditTrail.objects.count(), 0)

    def test_delete_obj_keep_audit_trail(self):
        """audit trail is kept after with obj it points at is deleted"""
        user = self.get_authenticated_user()
        create_user_audit_trail(user, USER_IP, self.obj)
        self.assertEqual(AuditTrail.objects.count(), 1)

        self.obj.delete()
        self.assertEqual(AuditTrail.objects.count(), 1)

        audit_trail = user.audittrail_set.all()[0]
        self.assertEqual(audit_trail.user, user)
        self.assertEqual(audit_trail.ip_address, USER_IP)
        self.assertIsNone(audit_trail.content_object)

    def test_delete_audit_trail(self):
        """audit trail deletion leaves other data untouched"""
        user = self.get_authenticated_user()
        create_user_audit_trail(user, USER_IP, self.obj)
        self.assertEqual(AuditTrail.objects.count(), 1)

        audit_trail = user.audittrail_set.all()[0]
        audit_trail.delete()
        
        UserModel.objects.get(id=user.id)
        UserModel.objects.get(id=self.obj.id)


class RemoveOldAuditTrailsTest(UserTestCase):
    def setUp(self):
        super(RemoveOldAuditTrailsTest, self).setUp()

        self.obj = UserModel.objects.create_user('BobBoberson', 'bob@example.com')
        
    def test_recent_audit_trail_is_kept(self):
        """remove_old_ips keeps recent audit trails"""
        user = self.get_authenticated_user()
        audit_trail = create_user_audit_trail(user, USER_IP, self.obj)

        remove_old_ips.send(None)

        self.assertEqual(user.audittrail_set.count(), 1)

    def test_old_audit_trail_is_removed(self):
        """remove_old_ips removes old audit trails"""
        user = self.get_authenticated_user()
        audit_trail = create_user_audit_trail(user, USER_IP, self.obj)

        audit_trail.created_on = timezone.now() - timedelta(days=50)
        audit_trail.save()

        remove_old_ips.send(None)

        self.assertEqual(user.audittrail_set.count(), 0)



from django import forms
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from .models import Agreement
from .utils import set_agreement_as_active


class AgreementForm(forms.ModelForm):
    type = forms.ChoiceField(label=_("Type"), choices=Agreement.TYPE_CHOICES)
    title = forms.CharField(
        label=_("Title"),
        help_text=_("Optional, leave empty for agreement to be named after its type."),
        required=False,
    )
    is_active = forms.BooleanField(
        label=_("Set as active for its type"),
        help_text=_(
            "If other agreement is already active for this type, it will be unset and replaced "
            "with this one. "
            "Misago will ask users who didn't accept this agreement to do so before allowing them "
            "to continue using the site's features."
        ),
        required=False,
    )
    link = forms.URLField(
        label=_("Link"),
        help_text=_("If your agreement is located on other page, enter here a link to it."),
        required=False,
    )
    text = forms.CharField(
        label=_("Text"),
        help_text=_("You can use Markdown syntax for rich text elements."),
        widget=forms.Textarea,
        required=False,
    )

    class Meta:
        model = Agreement
        fields = ['type', 'title', 'link', 'text', 'is_active']

    def clean(self):
        data = super(AgreementForm, self).clean()

        if not data.get('link') and not data.get('text'):
            raise forms.ValidationError(_("Please fill in agreement link or text."))

        return data

    def save(self):
        agreement = super(AgreementForm, self).save()
        if agreement.is_active:
            set_agreement_as_active(agreement)
        Agreement.objects.invalidate_cache()
        return agreement


class SearchAgreementsForm(forms.Form):
    type = forms.MultipleChoiceField(
        label=_("Type"),
        required=False,
        choices=Agreement.TYPE_CHOICES,
    )
    content = forms.CharField(
        label=_("Content"),
        required=False,
    )

    def filter_queryset(self, search_criteria, queryset):
        criteria = search_criteria
        if criteria.get('type') is not None:
            queryset = queryset.filter(type__in=criteria['type'])

        if criteria.get('content'):
            search_title = Q(title__icontains=criteria['content'])
            search_text = Q(text__icontains=criteria['content'])
            queryset = queryset.filter(search_title | search_text)

        return queryset



from hashlib import md5

from django.conf import settings
from django.utils.encoding import force_bytes

from misago.core.cache import cache
from misago.markup import common_flavour

from .models import Agreement, UserAgreement


def set_agreement_as_active(agreement, commit=False):
    agreement.is_active = True
    queryset = Agreement.objects.filter(type=agreement.type).exclude(pk=agreement.pk)
    queryset.update(is_active=False)

    if commit:
        agreement.save(update_fields=['is_active'])
        Agreement.objects.invalidate_cache()


def get_required_user_agreement(user, agreements):
    if user.is_anonymous:
        return None

    for agreement_type, _ in Agreement.TYPE_CHOICES:
        agreement = agreements.get(agreement_type)
        if agreement and agreement['id'] not in user.agreements:
            try:
                return Agreement.objects.get(id=agreement['id'])
            except Agreement.DoesNotExist:
                # possible stale cache
                Agreement.invalidate_cache()
    
    return None


def get_parsed_agreement_text(request, agreement):
    if not agreement.text:
        return None

    cache_name = 'misago_legal_%s_%s' % (agreement.pk, agreement.last_modified_on or '')
    cached_content = cache.get(cache_name)

    unparsed_content = agreement.text

    checksum_source = force_bytes('%s:%s' % (unparsed_content, settings.SECRET_KEY))
    unparsed_checksum = md5(checksum_source).hexdigest()

    if cached_content and cached_content.get('checksum') == unparsed_checksum:
        return cached_content['parsed']
    else:
        parsed = common_flavour(request, None, unparsed_content)['parsed_text']
        cached_content = {
            'checksum': unparsed_checksum,
            'parsed': parsed,
        }
        cache.set(cache_name, cached_content)
        return cached_content['parsed']


def save_user_agreement_acceptance(user, agreement, commit=False):
    user.agreements.append(agreement.id)
    UserAgreement.objects.create(agreement=agreement, user=user)

    if commit:
        user.save(update_fields=['agreements'])


#!/usr/bin/env python
# pylint: disable=E0401
"""
Creates a dev project for local development
"""

import os
import sys

BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

PROJECT_NAME = 'devproject'


def main():
    settings_file = os.path.join(BASE_DIR, PROJECT_NAME, PROJECT_NAME, 'settings.py')

    # Avoid recreating if already present
    if os.path.exists(settings_file):
        return

    from misago.core import setup

    setup.start_misago_project()
    fill_in_settings(settings_file)


def fill_in_settings(f):
    with open(f, 'r') as fd:
        s = fd.read()

        # Read PostgreSQL's config from env variables
        s = s.replace("'NAME': '',", "'NAME': os.environ.get('POSTGRES_DB'),")
        s = s.replace("'USER': '',", "'USER': os.environ.get('POSTGRES_USER'),")
        s = s.replace("'PASSWORD': '',", "'PASSWORD': os.environ.get('POSTGRES_PASSWORD'),")
        s = s.replace("'HOST': 'localhost',", "'HOST': os.environ.get('POSTGRES_HOST'),")

        # Specify console backend for email
        s += "\n"
        s += "\n# Set dev instance to send e-mails to console"
        s += "\n"
        s += "\nEMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'"
        s += "\n"

        # Tie Debug Toolbar visibility to env variable
        s += "\n"
        s += "\n# Display debug toolbar if IN_MISAGO_DOCKER enviroment var is set to \"1\""
        s += "\n"
        s += "\nDEBUG_TOOLBAR_CONFIG = {"
        s += "\n    'SHOW_TOOLBAR_CALLBACK': 'misago.conf.debugtoolbar.enable_debug_toolbar'"
        s += "\n}"
        s += "\n"

        # Empty the contents of STATICFILES_DIRS (STATICFILES_DIRS = [])
        pos = s.find('STATICFILES_DIRS')
        s = s[:s.find('[', pos) + 1] + s[s.find(']', pos):]

        # Remove theme dir from template dirs
        pos = s.find("'DIRS': [")
        s = s[:s.find('[', pos) + 1] + s[s.find(']', pos):]

    with open(f, 'w') as fd:
        fd.write(s)


if __name__ == '__main__':
    sys.argv.append(PROJECT_NAME)
    sys.path.append(BASE_DIR)
    main()


import os

from django.contrib.auth import get_user_model
from django.core.files import File
from django.urls import reverse

from misago.admin.testutils import AdminTestCase
from misago.users.datadownloads import request_user_data_download
from misago.users.models import DataDownload


UserModel = get_user_model()

TESTFILES_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'testfiles')
TEST_FILE_PATH = os.path.join(TESTFILES_DIR, 'avatar.png')


class DataDownloadAdminViewsTests(AdminTestCase):
    def test_link_registered(self):
        """admin nav contains data downloads link"""
        response = self.client.get(reverse('misago:admin:users:accounts:index'))

        response = self.client.get(response['location'])
        self.assertContains(response, reverse('misago:admin:users:data-downloads:index'))

    def test_list_view(self):
        """data downloads list view returns 200"""
        response = self.client.get(reverse('misago:admin:users:data-downloads:index'))
        self.assertEqual(response.status_code, 302)

        view_url = response['location']

        response = self.client.get(view_url)
        self.assertEqual(response.status_code, 200)

        request_user_data_download(self.user)
        response = self.client.get(view_url)
        self.assertEqual(response.status_code, 200)

    def test_expire_action(self):
        """expire action marks data download as expired and deletes its file"""
        data_download = request_user_data_download(self.user)

        with open(TEST_FILE_PATH, 'rb') as upload:
            data_download.file = File(upload)
            data_download.save()

        self.assertIsNotNone(data_download.file)
        self.assertTrue(os.path.isfile(data_download.file.path))

        response = self.client.post(
            reverse('misago:admin:users:data-downloads:index'),
            data={
                'action': 'expire',
                'selected_items': [data_download.pk],
            },
        )
        self.assertEqual(response.status_code, 302)

        updated_download = DataDownload.objects.get(pk=data_download.pk)
        self.assertEqual(updated_download.status, DataDownload.STATUS_EXPIRED)
        self.assertFalse(updated_download.file)

        self.assertFalse(os.path.isfile(data_download.file.path))

    def test_delete_action(self):
        """dele action deletes data download together with its file"""
        data_download = request_user_data_download(self.user)

        with open(TEST_FILE_PATH, 'rb') as upload:
            data_download.file = File(upload)
            data_download.save()

        self.assertIsNotNone(data_download.file)
        self.assertTrue(os.path.isfile(data_download.file.path))

        response = self.client.post(
            reverse('misago:admin:users:data-downloads:index'),
            data={
                'action': 'delete',
                'selected_items': [data_download.pk],
            },
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(DataDownload.objects.count(), 0)
        self.assertFalse(os.path.isfile(data_download.file.path))

    def test_request_view(self):
        """request data downloads view initializes new downloads"""
        response = self.client.get(reverse('misago:admin:users:data-downloads:request'))
        self.assertEqual(response.status_code, 200)

        other_user = UserModel.objects.create_user('bob', 'bob@boberson.com')

        response = self.client.post(
            reverse('misago:admin:users:data-downloads:request'),
            data={
                'user_identifiers': '\n'.join([
                    self.user.username,
                    other_user.email,
                ]),
            },
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(DataDownload.objects.count(), 2)

    def test_request_view_empty_data(self):
        """request data downloads view handles empty data"""
        response = self.client.get(reverse('misago:admin:users:data-downloads:request'))
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            reverse('misago:admin:users:data-downloads:request'),
            data={'user_identifiers': ''},
        )
        self.assertEqual(response.status_code, 200)

        self.assertEqual(DataDownload.objects.count(), 0)

    def test_request_view_user_not_found(self):
        """request data downloads view handles empty data"""
        response = self.client.get(reverse('misago:admin:users:data-downloads:request'))
        self.assertEqual(response.status_code, 200)

        response = self.client.post(
            reverse('misago:admin:users:data-downloads:request'),
            data={'user_identifiers': 'not@found.com'},
        )
        self.assertEqual(response.status_code, 200)

        self.assertEqual(DataDownload.objects.count(), 0)


from django.core import mail as djmail
from django.template.loader import render_to_string
from django.utils.translation import get_language

from misago.conf import db_settings, settings

from .utils import get_host_from_address


def build_mail(recipient, subject, template, sender=None, context=None):
    context = context.copy() if context else {}
    context.update({
        'SITE_ADDRESS': settings.MISAGO_ADDRESS,
        'SITE_HOST': get_host_from_address(settings.MISAGO_ADDRESS),
        'LANGUAGE_CODE': get_language()[:2],
        'LOGIN_URL': settings.LOGIN_URL,

        'misago_settings': db_settings,

        'user': recipient,
        'sender': sender,
        'subject': subject,
    })

    message_plain = render_to_string('%s.txt' % template, context)
    message_html = render_to_string('%s.html' % template, context)

    message = djmail.EmailMultiAlternatives(subject, message_plain, to=[recipient.email])
    message.attach_alternative(message_html, "text/html")

    return message


def mail_user(recipient, subject, template, sender=None, context=None):
    message = build_mail(recipient, subject, template, sender, context)
    message.send()


def mail_users(recipients, subject, template, sender=None, context=None):
    messages = []

    for recipient in recipients:
        messages.append(build_mail(recipient, subject, template, sender, context))

    if messages:
        send_messages(messages)


def send_messages(messages):
    connection = djmail.get_connection()
    connection.send_messages(messages)



from django.conf.urls import url
from django.utils.translation import ugettext_lazy as _

from .views.admin import (
    AgreementsList, DeleteAgreement, EditAgreement, NewAgreement, SetAgreementAsActive
)


class MisagoAdminExtension(object):
    def register_urlpatterns(self, urlpatterns):
        # Legal Agreements
        urlpatterns.namespace(r'^agreements/', 'agreements', 'users')
        urlpatterns.patterns(
            'users:agreements',
            url(r'^$', AgreementsList.as_view(), name='index'),
            url(r'^(?P<page>\d+)/$', AgreementsList.as_view(), name='index'),
            url(r'^new/$', NewAgreement.as_view(), name='new'),
            url(r'^edit/(?P<pk>\d+)/$', EditAgreement.as_view(), name='edit'),
            url(r'^delete/(?P<pk>\d+)/$', DeleteAgreement.as_view(), name='delete'),
            url(r'^set-as-active/(?P<pk>\d+)/$', SetAgreementAsActive.as_view(), name='set-as-active'),
        )
        
    def register_navigation_nodes(self, site):
        site.add_node(
            name=_("Agreements"),
            icon='fa fa-check-square-o',
            parent='misago:admin:users',
            after='misago:admin:users:data-downloads:index',
            namespace='misago:admin:users:agreements',
            link='misago:admin:users:agreements:index',
        )


from django.core.management import BaseCommand

from misago.conf import settings
from misago.users.signals import remove_old_ips


class Command(BaseCommand):
    help = "Removes users IPs stored for longer than set in MISAGO_IP_STORE_TIME."
    
    def handle(self, *args, **options):
        if not settings.MISAGO_IP_STORE_TIME:
            self.stdout.write("Old IP removal is disabled.")
            return

        remove_old_ips.send(sender=self)

        self.stdout.write(
            "IP addresses older than {} days have been removed.".format(settings.MISAGO_IP_STORE_TIME))
